﻿/* **************************************************************************
 *
 * Copyright 2007-2010 Jeff Hardy
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * *************************************************************************/

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using Microsoft.Web.Management.Client;
using Microsoft.Web.Management.Client.Win32;

namespace NWSGI.ManagementClient
{
    class ScriptMappingsPage : ModuleListPage, IModuleChildPage
    {
        public IModulePage ParentPage { get; set; }
        protected NwsgiModuleServiceProxy ServiceProxy { get; private set; }

        private ScriptMappingItem SelectedItem
        {
            get { return ListView.SelectedItems.Count > 0 ? (ScriptMappingItem)ListView.SelectedItems[0] : null; }
        }

        private PageTaskList taskList;
        protected override TaskListCollection Tasks
        {
            get
            {
                if(this.taskList == null)
                {
                    this.taskList = new PageTaskList(this);
                }

                TaskListCollection col = base.Tasks;
                col.Add(this.taskList);
                return col;
            }
        }

        protected override bool CanRefresh
        {
            get { return !base.InProgress; }
        }

        protected override void InitializeListPage()
        {
            string[] headers = new string[] { "Script Name", "Physical Path", "Callable" };

            foreach(string headerTitle in headers)
            {
                ColumnHeader header = new ColumnHeader();
                header.Text = headerTitle;
                header.Width = -2;
                ListView.Columns.Add(header);
            }

            ListView.LabelEdit = false;
            ListView.MultiSelect = false;

            ListView.ItemActivate += OnListViewItemActivate;
            ListView.SelectedIndexChanged += OnListViewSelectedIndexChanged;
            ListView.KeyUp += OnListViewKeyUp;
        }

        protected override void OnActivated(bool initialActivation)
        {
            base.OnActivated(initialActivation);
            if(initialActivation)
            {
                GetItemsAsync();
                ServiceProxy = (NwsgiModuleServiceProxy)CreateProxy(typeof(NwsgiModuleServiceProxy));
            }
        }

        protected override void Refresh()
        {
            if(CanRefresh)
                GetItemsAsync();
        }

        internal void GoBack()
        {
            base.Navigate(typeof(NwsgiPage));
        }

        private void GetItemsAsync()
        {
            Cursor.Current = Cursors.AppStarting;
            StartAsyncTask("Getting variables...", new DoWorkEventHandler(GetItems),
                                                   new RunWorkerCompletedEventHandler(GetItemsCompleted));
        }

        private void GetItems(object sender, DoWorkEventArgs e)
        {
            ArrayList variables = ServiceProxy.GetScriptMappings();
            List<ScriptMappingInfo> items = new List<ScriptMappingInfo>();

            foreach(string[] variable in variables)
            {
                items.Add(new ScriptMappingInfo(variable));
            }

            e.Result = items;
        }

        private void GetItemsCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ListView.BeginUpdate();
            ListView.Items.Clear();
            try
            {
                IEnumerable<ScriptMappingInfo> variables = (IEnumerable<ScriptMappingInfo>)e.Result;
                foreach(ScriptMappingInfo value in variables)
                {
                    ListView.Items.Add(new ScriptMappingItem(value));
                }
            }
            finally
            {
                base.Update();
                ListView.EndUpdate();
                Cursor.Current = Cursors.Default;
            }
        }

        private void OnListViewSelectedIndexChanged(object sender, EventArgs e)
        {
            base.Update();
        }

        private void OnListViewKeyUp(object sender, KeyEventArgs e)
        {
            if(this.SelectedItem != null && e.KeyData == Keys.Delete)
                this.DeleteItem();
        }

        private void OnListViewItemActivate(object sender, EventArgs e)
        {
            if(this.SelectedItem != null)
            {
                this.EditItem();
            }
        }

        private void AddItem(ScriptMappingInfo entry, bool isSelected)
        {
            ScriptMappingItem item = new ScriptMappingItem(entry);
            base.ListView.Items.Add(item);

            if(isSelected)
            {
                item.Selected = true;
                item.Focused = true;
                base.ListView.EnsureVisible(item.Index);
            }
        }

        internal void AddScriptMapping()
        {
            using(ScriptMappingEditor form = new ScriptMappingEditor(base.Module, this.ServiceProxy))
            {
                if(base.ShowDialog(form) == DialogResult.OK)
                {
                    AddItem(form.Entry, true);
                }
            }
        }

        private void ReplaceItem(ScriptMappingItem item, ScriptMappingInfo entry)
        {
            base.ListView.Items.Remove(item);
            this.AddItem(entry, true);
        }

        internal void EditItem()
        {
            ScriptMappingItem selectedItem = this.SelectedItem;
            if(selectedItem != null)
            {
                using(ScriptMappingEditor form = new ScriptMappingEditor(base.Module, this.ServiceProxy, selectedItem.Entry))
                {
                    if((base.ShowDialog(form) == DialogResult.OK) && form.HasChanges)
                    {
                        this.ReplaceItem(selectedItem, form.Entry);
                    }
                }
            }
        }

        internal void DeleteItem()
        {
            ScriptMappingItem selectedItem = this.SelectedItem;
            if(selectedItem != null)
            {
                ScriptMappingInfo entry = selectedItem.Entry;
                if(base.ShowMessage(string.Format("Are you sure you want to remove the script mapping for '{0}'? This cannot be undone.", new object[] { entry.ScriptName }), MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
                {
                    try
                    {
                        Cursor.Current = Cursors.WaitCursor;
                        this.ServiceProxy.DeleteScriptMapping(entry);
                        base.ListView.Items.Remove(selectedItem);
                    }
                    catch(Exception exception)
                    {
                        base.DisplayErrorMessage(exception, Resources.ResourceManager);
                    }
                    finally
                    {
                        Cursor.Current = Cursors.Default;
                    }
                }
            }
        }

        private sealed class PageTaskList : TaskList
        {
            private ScriptMappingsPage owner;

            public PageTaskList(ScriptMappingsPage owner)
            {
                this.owner = owner;
            }

            public override ICollection GetTaskItems()
            {
                ArrayList list = new ArrayList();

                list.Add(new MethodTaskItem("AddItem", "Add...", "Actions", "Add a new variable"));

                ListViewItem selected = this.owner.SelectedItem;
                if(selected != null)
                {
                    list.Add(new MethodTaskItem("EditItem", "Edit", "Actions", "Edit the currently selected variable"));
                    list.Add(new MethodTaskItem("DeleteItem", "Remove", "Actions", "Delete the currently selected variable"));
                }

                list.Add(new MethodTaskItem("GoBack", "Go Back", "Tasks", "Go back to the parent page"));

                return list;
            }

            public void AddItem()
            {
                this.owner.AddScriptMapping();
            }

            public void EditItem()
            {
                this.owner.EditItem();
            }

            public void DeleteItem()
            {
                this.owner.DeleteItem();
            }

            public void GoBack()
            {
                this.owner.GoBack();
            }
        }

        private sealed class ScriptMappingItem : ListViewItem
        {
            public ScriptMappingInfo Entry { get; private set; }

            public ScriptMappingItem(ScriptMappingInfo entry) : base(entry.ToArray())
            {
                Entry = entry;
            }
        }
    }

    internal class ScriptMappingInfo
    {
        public string ScriptName { get; set; }
        public string PhysicalPath { get; set; }
        public string Callable { get; set; }

        public ScriptMappingInfo() { }

        public ScriptMappingInfo(string[] values)
        {
            ScriptName = values[0];
            PhysicalPath = values[1];
            Callable = values[2];
        }

        public string[] ToArray()
        {
            return new string[] { ScriptName, PhysicalPath, Callable };
        }
    }
}

