﻿/* **************************************************************************
 *
 * Copyright 2007-2010 Jeff Hardy
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * *************************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Xunit;
using IronPython.Hosting;
using IronPython.Runtime;
using Moq;

namespace NWSGI.Tests
{
    public class StartResponseFacts
    {
        private StubConfig config;
        private Microsoft.Scripting.Hosting.ScriptEngine engine;

        public StartResponseFacts()
        {
            this.config = new StubConfig();

            this.engine = Python.CreateEngine();
        }
        
        [Fact]
        public void Should_return_write_callable()
        {
            var wsgi = CreateExecutor(new StubResponse());

            wsgi.Run(new WsgiCallableDelegate(delegate(PythonDictionary environ, StartResponseDelegate start_response) {
                var write = start_response("200 OK", new List(), null);

                Assert.NotNull(write);

                return Enumerable.Empty<string>();
            }));

            //wsgi.Run(new WsgiCallableDelegate((environ, start_response) =>
            //{
            //    var write = start_response("200 OK", new List(), null);

            //    Assert.NotNull(write);

            //    return Enumerable.Empty<string>();
            //}));
        }

        [Fact]
        public void Should_set_response_status()
        {
            var response = new Mock<IWsgiResponse>();
            var wsgi = CreateExecutor(response.Object);

            wsgi.Run((environ, start_response) =>
            {
                start_response("200 OK", new List(), null);

                return Enumerable.Empty<string>();
            });

            response.Verify(r => r.SetStatus("200 OK"), "Response status was not set");
        }

        [Fact]
        public void Should_set_response_header()
        {
            var response = new Mock<IWsgiResponse>();
            var wsgi = CreateExecutor(response.Object);

            wsgi.Run((environ, start_response) =>
            {
                var headers = new List();
                headers.append(new PythonTuple(new[] { "Content-Length", "0" }));

                start_response("200 OK", headers, null);

                return Enumerable.Empty<string>();
            });

            response.Verify(r => r.AddHeader("Content-Length", "0"), "Header was not set");
        }

        [Fact]
        public void Should_set_response_headers()
        {
            var response = new Mock<IWsgiResponse>();
            var wsgi = CreateExecutor(response.Object);

            wsgi.Run((environ, start_response) =>
            {
                var headers = new List();
                headers.append(new PythonTuple(new[] { "Content-Length", "0" }));
                headers.append(new PythonTuple(new[] { "Content-Type", "text/plain" }));

                start_response("200 OK", headers, null);

                return Enumerable.Empty<string>();
            });

            response.Verify(r => r.AddHeader("Content-Length", "0"), "Content-Length header was not set");
            response.Verify(r => r.AddHeader("Content-Type", "text/plain"), "Content-Type header was not set");
        }

        [Fact]
        public void Should_not_set_headers_if_none_provided()
        {
            var response = new Mock<IWsgiResponse>();
            var wsgi = CreateExecutor(response.Object);

            wsgi.Run((environ, start_response) =>
            {
                start_response("200 OK", new List(), null);

                return Enumerable.Empty<string>();
            });

            response.Verify(r => r.AddHeader(It.IsAny<string>(), It.IsAny<string>()), Times.Never());
        }

        private WsgiExecutor CreateExecutor(IWsgiResponse response)
        {
            var context = new StubContext
            {
                Request = new StubRequest(),
                Response = response
            };

            return new WsgiExecutor(engine, config, context);
        }
    }
}
